local L = AceLibrary("AceLocale-2.2"):new("Omen")
local dewdrop = AceLibrary("Dewdrop-2.0")
local deformat = AceLibrary("Deformat-2.0")
local sprintf = string.format

function Omen_CreateTempTableFuncs()
	local tempTables = {}
	
	local clearFunc = function(table)
		for k in pairs(table) do table[k] = nil end
	end
	
	local getFunc = function (name, clear)	
		local n = name:lower()
		tempTables[n] = tempTables[n] or {}
		t = tempTables[n]
		if clear then
			clearFunc(t)
		end
		return t
	end
	return getFunc, clearFunc
end

local get_table, clear_table = Omen_CreateTempTableFuncs()

Omen = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceComm-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1", "FuBarPlugin-2.0", "AceDebug-2.0", "AceModuleCore-2.0")
Omen:SetModuleMixins("AceDebug-2.0", "AceEvent-2.0", "AceModuleCore-2.0")

Omen.commPrefix = "OmenThreatMonitor"
Omen.OnCommReceive = {}

Omen.ThreatConstants = {
	["healing"] = 0.5,
	["meleeaggrogain"] = 1.1,
	["rangeaggrogain"] = 1.3,
	["ragegain"] = 5.0,
	["energygain"] = 5.0,
	["managain"] = 0.5,
}

function Omen:OnInitialize()
	self.BuffModifiers = OmenBuffModifiers

	self:ToggleModuleActive("ClassCore", true)
	self.ClassModules = self:GetModule("ClassCore")

	-- self:ToggleModuleActive("NPCCore", true)
	-- self.NPCModules = self:GetModule("NPCCore")
	
	self:SetCommPrefix(Omen.commPrefix)
	self:RegisterMemoizations({
		"THREAT_UPDATE",
		"BOSS_EVENT",
		"CLEAR_REQUEST",
		"SET_MASTER_TARGET",
		"GENERIC_TARGET"
	})
	self:RegisterComm(self.commPrefix, "GROUP", "OnCommReceive")
	
	self.threatLevels = {}
	self.threatHistory = {}
	self.MaxHistory = 100
	--[[
	AceLibrary("AceComm-2.0"):QueryAddonVersion(self.commPrefix, "GROUP")
	
	AceLibrary("AceComm-2.0"):RegisterAddonVersionReceptor(function(player, addon, version)
	    if version then
	        if version == true then
	            Omen:Print("Player %s has %s", player, addon)
	        else
	            Omen:Print("Player %s has %s version %s", player, addon, version)
	        end
	    else
	        Omen:Print("Player %s does not have %s", player, addon)
	    end
	end)
	]]--
	
end

-- Command handling
function Omen.OnCommReceive:THREAT_UPDATE(prefix, sender, distribution, targetThreat, generalThreat, threatPerSec)
	-- We're going storing a flattened list of participants and their threat values in a single table
	self.threatTargets[sender] = self.threatTargets[sender] or get_table("threat_targets_" .. sender, true)
	
	tinsert(self.threatTargets[sender], targetThreat)
	tinsert(self.threatTargets[sender], generalThreat)
	tinsert(self.threatTargets[sender], targetThreat + generalThreat)
	tinsert(self.threatTargets[sender], threatPerSec)
	
	self:Debug(sprintf("Threat levels for %s changed to %s (%s general, %s target, %s TPS)", sender, (targetThreat + generalThreat), generalThreat, targetThreat, threatPerSec))
end

function Omen.OnCommReceive:CLEAR_REQUEST(amount)
	self.ClassModules:ClearThreat()
	self:Debug("Threat levels cleared")
end

-- Command sending
function Omen:CancelThreatPublish()
	if not self:IsEventScheduled("OmenThreatPublish") then return end
	self:CancelScheduledEvent("OmenThreatPublish")
end

function Omen:PublishThreat()
	if self:IsEventScheduled("OmenThreatPublish") then return end
	self:ScheduleRepeatingEvent("OmenThreatPublish", function()
		local targetThreat, generalThreat, threatPerSec = Omen.ClassModules:GetThreat()
		local totalThreat = targetThreat + generalThreat
		if Omen.lastSentThreat ~= totalThreat then
			Omen.lastSentThreat = totalThreat
			Omen:SendCommMessage("GROUP", "THREAT_UPDATE", targetThreat, generalThreat, threatPerSec)
		end
	end, 0.5)	
end

-- My displayz, let me show you them
function Omen:UpdateDisplay()
end